/*
 *  OOo Dictionary Installer (c) Jonathan Hudson 2002.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <utime.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include "callbacks.h"
#include "ooodi.h"
#include <assert.h>

static char *afx[] = {".dic", ".aff"};


static void get_iter_from_n(GtkTreeModel*model, int n, GtkTreeIter *iter)
{
    char gpath[16];
    snprintf(gpath, sizeof(gpath),"%d", n);
    gtk_tree_model_get_iter_from_string(model, iter, gpath);
}


static void set_busy_cursor(int flg)
{
    static GdkCursor *watch;
    if(watch == NULL)
    {
        watch = gdk_cursor_new(GDK_WATCH);
    }
    if(flg)
    {
        gdk_window_set_cursor(gblptr->window->window, watch);
    }
    else
    {
        gdk_window_set_cursor(gblptr->window->window, NULL);
    }
}

static inline char *make_url(char *url, char *name)
{
    return buildpath(url, gblptr->uri, name, NULL);
}

static inline char *make_local(char *file, char *name)
{
    return buildpath(file, gblptr->basedir, name, NULL);
}

void
on_window1_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
    gtk_exit(0);
}

void
on_dlcancel_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{

    DLCB_t *p;
    p = gtk_object_get_user_data (GTK_OBJECT(button));
    p->flag = CANERR;
}


static int index_in_list(DICTLST_t *p, char *region, char *spec, int type)
{
    int i;
    int idx = -1;

    for(i = 0; i < p->total; i++)
    {
        if((p->item[i].type == type)  &&
           0 == strcmp(p->item[i].spec, spec) &&
           0 == strcmp(p->item[i].region, region))
        {
            idx = i;
            break;
        }
    }
    return idx;
}


static void remove_dict_files (char *path, char *spec, int location)
{
    char *q,*p;
    int j, nref = 0;

        /* Only remove if this is a unique reference */

    for(j = 0; j < gblptr->avail.total; j++)
    {
        if((gblptr->avail.item[j].flag & (1 << location))
           && (0 == strcmp(gblptr->avail.item[j].spec, spec)))
        {
            nref++;
        }
    }

    if(0 == nref)
    {
        p = strrchr(path, '/');
        p++;

        for(j = 0; j < 2; j++)
        {
            q = stpcpy(p, spec);
            strcpy(q, afx[j]);
            unlink(path);
        }
        *p = 0;
    }
}

static char *add_special_entry (DICTLST_t *p, int i, FILE *fp)
{
    char *pdot;
    char *aspec;
    
    fprintf(fp, "DICT " "%s %s %s\n",
            p->item[i].lang, p->item[i].region, p->item[i].spec);
                        
    aspec = g_strdup(p->item[i].file);
    if((pdot = strchr(aspec, '.')) != NULL)
    {
        char path[PATH_MAX], oaff[PATH_MAX],naff[PATH_MAX];        
        *pdot = '\0';
        if(NULL != getcwd(path, sizeof(path)))
        {
            snprintf(oaff, sizeof(oaff), "%s.aff", aspec);
            snprintf(naff, sizeof(naff), "%s.aff", p->item[i].spec);
            chdir(gblptr->avail.path);
            unlink(naff);
            symlink(oaff, naff);
            chdir(path);
        }
    }
    return aspec;
}

static void update_dictfile(DICTLST_t *p, int action, int location)
{
    FILE *ofp, *nfp;
    char *q, file[PATH_MAX],tmp_file[PATH_MAX], dict_file[PATH_MAX];
    int i;
    
    char *pth;
    GValue gv = {0};    
    GtkTreeSelection *select;
    GtkTreeModel*model;
    GtkTreeIter iter;

    select = gtk_tree_view_get_selection(GTK_TREE_VIEW (gblptr->avail.clist));
    model = gtk_tree_view_get_model (GTK_TREE_VIEW (gblptr->avail.clist));

    g_value_init (&gv, G_TYPE_BOOLEAN);    
    pth = (location == USER_LOCATION) ? p->path : p->path_shr;
    
    q = stpcpy(file, pth);
    strcpy(q, "/" DICTLST);

    q = stpcpy(tmp_file, pth);
    strcpy(q, "/." DICTLST);

    ofp = fopen(file, "r");

    q = stpcpy(dict_file, pth);
    *q++ = '/';
    *q = '\0';
    
    if(NULL != (nfp = fopen(tmp_file, "w")))
    {
        if(NULL != ofp)
        {
            char buf[128];
            int type;

            while(fgets(buf, sizeof(buf), ofp))
            {
                char dtyp[5], *spec, *region;
                int  justcopy = 1;
                
                    /*
                     * Get the DICT values and process as:
                     * Find in available.lst
                     * if the file doesn't exist, the remove from file
                     * if the file exists, then just clear the new flag
                     */
                if(3 == sscanf(buf, "%4s %*s %as %as", dtyp, &region, &spec))
                {
                    if(strcmp(dtyp, DICT) == 0)
                    {
                        type = IS_DICT;
                    }
                    else if(strcmp(dtyp, HYPH) == 0)
                    {
                        type = IS_HYPH;
                    }
                    else
                    {
                        type = -1;
                    }
                    
                    if(-1 != type && -1 != (i = index_in_list(p, region,
                                                              spec, type)))
                    {
                        switch(action)
                        {
                            case ADD_NEW:
                                p->item[i].flag &= ~DICT_AVAILABLE;
                                if(p->item[i].flag & DICT_SPECIAL_LINK)
                                {
                                    add_special_entry(p, i, nfp);
                                    fgets(buf, sizeof(buf), ofp);
                                }
                                else
                                {
                                    snprintf (q, (size_t)(q-dict_file), "%s.dic", spec);

                                    if(access(dict_file, R_OK) != 0)
                                    {
                                        justcopy = 0;
                                    }
                                }
                                break;
                            case DEL_DIC:
                                if(p->item[i].flag & DICT_SELECTED)
                                {

                                    if(p->item[i].flag & DICT_SPECIAL_LINK)
                                    {
                                        fgets(buf, sizeof(buf), ofp);
                                    }

                                    get_iter_from_n(model, i, &iter);
                                    gtk_tree_selection_unselect_iter (select, &iter);
                                    g_value_set_boolean(&gv, 0);
                                    gtk_list_store_set_value (GTK_LIST_STORE(model), &iter, location, &gv);
                                    
                                    p->item[i].flag &= ~(DICT_SELECTED|(1 << location));

                                    if(gblptr->unlink_on_remove)
                                    {
                                        remove_dict_files(dict_file, spec, location);
                                    }
                                    justcopy = 0;
                                }
                                break;
                            default:
                                break;
                        }
                        free(spec);
                        free(region);
                    }
                }

                if(justcopy)
                {
                    fputs(buf, nfp);
                }
            }
            fclose(ofp);
        }
        
            /*
             * Handle any NEW or bogus entries
             *  (inter alia the el_GR/en_GR_enhanced issue
             */

        for(i = 0; i < p->total; i++)
        {
            if(action == ADD_NEW)
            {
                if(p->item[i].flag &  DICT_AVAILABLE)
                {
                    char *aspec;
                    if(p->item[i].flag & DICT_SPECIAL_LINK)
                    {
                        aspec = add_special_entry(p, i, nfp);
                    }
                    else
                    {
                        aspec =  g_strdup(p->item[i].spec);
                    }
                    fprintf(nfp, "%s %s %s %s\n",
                            (p->item[i].type == IS_DICT) ? DICT : HYPH,
                            p->item[i].lang,
                            p->item[i].region,                            
                            aspec);

                    get_iter_from_n(model, i, &iter);
                    g_value_set_boolean(&gv, 1);
                    gtk_list_store_set_value (GTK_LIST_STORE(model), &iter, location, &gv);

                    p->item[i].flag &= ~DICT_AVAILABLE;
                    p->item[i].flag |= (1 << location);
                    g_free(aspec);
                }
            }
            else if(action == DEL_DIC)
            {
                if(p->item[i].flag & DICT_SELECTED)
                {
                    p->item[i].flag &= ~(DICT_SELECTED|(1 << location));

                    get_iter_from_n(model, i, &iter);
                    gtk_tree_selection_unselect_iter (select, &iter);
                    g_value_set_boolean(&gv, 1);
                    gtk_list_store_set_value (GTK_LIST_STORE(model), &iter, location, &gv);

                    if(gblptr->unlink_on_remove)
                    {
                        remove_dict_files(dict_file, p->item[i].spec, location);
                    }
                }
            }
        }
        fclose(nfp);
    }

    if(nfp)
    {
        if(ofp)
        {
            strcpy(q, "." DICTLST ".old");
            rename(file, dict_file);
        }
        rename(tmp_file, file);
    }
}

void
on_installbutton_clicked                  (GtkButton       *button,
                                           gpointer         user_data)
{
    DICTLST_t *p;
    GtkWidget *text;
    GtkWidget *cancel;
    static short ticon;
    GtkTreeSelection *select;
    GtkTreeModel *model;
    GtkTreeIter iter;
    int user_flag;

    int i;
    int nerr = 0;

    user_flag = (int) gtk_object_get_user_data (GTK_OBJECT(button));

    
    model = gtk_tree_view_get_model (GTK_TREE_VIEW (gblptr->avail.clist));
    select = gtk_tree_view_get_selection(GTK_TREE_VIEW (gblptr->avail.clist));
    
    p = &gblptr->avail;
    assert(NULL != (text = glade_xml_get_widget (xml, "errtext")));
    gtk_widget_hide_on_delete (gblptr->errwin);
    assert(NULL != (cancel = glade_xml_get_widget (xml, "errbutton")));
    gtk_object_set_user_data (GTK_OBJECT(cancel), gblptr->errwin);
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW(text)), "", 0);

    for(i = 0; i < p->total; i++)
    {
        if(p->item[i].flag &  DICT_SELECTED)
        {
            char url[PATH_MAX], dlfile[PATH_MAX];
            char *txt = NULL;
            int res = -1;

            make_local(dlfile, p->item[i].file);
            
            if(gblptr->uselocal && (access(dlfile, R_OK) == 0))
            {
                get_iter_from_n(model, i, &iter);
                gtk_tree_model_get(model, &iter, 3, &txt, -1);
                    /* Only use the local file if we haven't
                     *  found a newer file on the 'net
                     */
                if(!strchr(txt,'*'))
                {
                    res = 0;                    
                }
                g_free(txt);
            }
            
            if(res != 0)
            {
                make_url(url, p->item[i].file);
                res = OODownload(url, dlfile, ZIP_CHECK);
            }
            
            if(0 == res)
            {
                p->item[i].flag |=  DICT_DOWNLOADED;
                get_iter_from_n(model, i, &iter);
                gtk_tree_selection_unselect_iter (select, &iter);
            }
            else
            {
                nerr++;
            }
        }
    }

    if(nerr)
    {
        gtk_widget_show (gblptr->errwin);
        if(0 == ticon)
        {
            ticon = 1;
            setwinicon (gblptr->errwin);
        }
    }
    
    for(i = 0; i < p->total; i++)
    {
        if(p->item[i].flag &  DICT_DOWNLOADED)
        {
            char cmd[PATH_MAX];
            char dlfile[PATH_MAX];
            
            int res;
            char *pth;

            make_local(dlfile, p->item[i].file);            

            pth = (user_flag == USER_LOCATION) ? p->path : p->path_shr;

            snprintf(cmd, sizeof(cmd), UNZIPCMDD, dlfile, pth);

            res = system(cmd);
            if(WEXITSTATUS(res) == 0)
            {
                char *txt, *pstr;
                
                p->item[i].flag &= ~DICT_DOWNLOADED;
                p->item[i].flag |=  DICT_AVAILABLE;

                get_iter_from_n(model, i, &iter);
                gtk_tree_model_get(model, &iter, TYPE_COL, &txt, -1);

                if((pstr = strchr(txt,'*')) != NULL)
                {
                    GValue gv = {0};
                    g_value_init (&gv, G_TYPE_STRING);
                    
                    *pstr = '\0';

                    g_value_set_string (&gv, txt);
                    gtk_list_store_set_value (GTK_LIST_STORE(model), &iter, TYPE_COL, &gv);
                    g_free(txt);
                }
                if(gblptr->keepzips == 0)
                {
                    unlink(dlfile);
                }
            }
        }
    }
    update_dictfile(p, ADD_NEW, user_flag);
}

void
on_errordialog_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
    gtk_widget_hide(GTK_WIDGET(object));
}


void
on_errbutton_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *p;
    p = gtk_object_get_user_data (GTK_OBJECT(button));
    gtk_widget_hide(p);
}


void
on_cancelbutton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_main_quit();
}


void
on_removebutton_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    int user_flag = (int) gtk_object_get_user_data (GTK_OBJECT(button));
    update_dictfile(&gblptr->avail, DEL_DIC, user_flag);
}

static void activate_prefs()
{
    GtkWidget *t;
    if(gblptr->proxy)
    {
        assert(NULL != (t = glade_xml_get_widget (xml, "propProxytext")));
        gtk_entry_set_text(GTK_ENTRY(t), gblptr->proxy);
    }
    
    assert(NULL != (t = glade_xml_get_widget (xml, "propURItext")));
    gtk_entry_set_text(GTK_ENTRY(t), gblptr->uri);

    assert(NULL != (t = glade_xml_get_widget (xml, "proppxuser")));
    gtk_entry_set_text(GTK_ENTRY(t), gblptr->pxuser);

    assert(NULL != (t = glade_xml_get_widget (xml, "propbrowser")));
    gtk_entry_set_text(GTK_ENTRY(t), gblptr->browser);

    if(strcmp(".", gblptr->basedir) != 0)
    {
        assert(NULL != (t = glade_xml_get_widget (xml, "propbasedir")));
        gtk_entry_set_text(GTK_ENTRY(t), gblptr->basedir);
    }
    
    assert(NULL != (t = glade_xml_get_widget (xml, "propsremovecheck")));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t), gblptr->unlink_on_remove);
    assert(NULL != (t = glade_xml_get_widget (xml, "propzips")));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t), gblptr->keepzips);
    assert(NULL != (t = glade_xml_get_widget (xml, "proplocal")));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t), gblptr->uselocal);
    assert(NULL != (t = glade_xml_get_widget (xml, "prophyph")));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t), gblptr->usehyph);

    gtk_widget_show(gblptr->props);
}


void
on_properties_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    activate_prefs();
}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *w;
    assert(NULL != (w = glade_xml_get_widget (xml, "verslabel")));
    gtk_label_set_text(GTK_LABEL(w),"Version " VERSION ", " __DATE__);

    assert(NULL != (w = glade_xml_get_widget (xml, "urlbutton")));
    
    if(!(gblptr->browser && *gblptr->browser))
    {
        gtk_widget_set_sensitive (w, 0);
    }
    gtk_widget_show(gblptr->about);

}


void
on_popquit_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gtk_exit(0);
}


gboolean
on_ooo_window_button_press_event       (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    if (event->type == GDK_BUTTON_PRESS) {
        GdkEventButton *bevent = (GdkEventButton *) event;
        if(3 == bevent->button)
        {
            gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL,
                            bevent->button, bevent->time);
            return TRUE;
        }
    }
    return FALSE;
}

void
on_aboutbutton_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *p;
    assert(NULL != (p = glade_xml_get_widget (xml, "about")));
    gtk_widget_hide(p);
}

void
on_about_destroy                       (GtkObject       *object,
                                        gpointer         user_data)
{
    gtk_widget_hide(GTK_WIDGET(object));
}


void
on_props_destroy                       (GtkObject       *object,
                                        gpointer         user_data)
{
    gtk_widget_hide(GTK_WIDGET(object));
}


void
on_propcancel_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *p;
    assert(NULL != (p = glade_xml_get_widget (xml, "props")));
    gtk_widget_hide(p);
}

static void check_for_change(char **prop, char *newval, int *chg)
{
    if(strcmp(newval, *prop))
    {
        g_free(*prop);
        *prop = newval;
        (*chg)++;
    }
    else
    {
        g_free(newval);
    }
}

static int check_for_toggle(char *prop, int newval, int *chg)
{
    int res;
    
    if((res = (*prop != newval)))
    {
        *prop = newval;
        (*chg)++;
    }
    return res;
}

void
on_propOK_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *p;
    GtkWidget *t;
    char *pp, *pu, *px, *pb, *pd;
    int rem, zip, loc, hyph;
    int chg = 0;

    GtkTreeModel*model;
    GtkTreeIter iter;

    model = gtk_tree_view_get_model (GTK_TREE_VIEW (gblptr->avail.clist));
    
    assert(NULL != (p = glade_xml_get_widget (xml, "props")));

    assert(NULL != (t = glade_xml_get_widget (xml, "propProxytext")));
    pp = gtk_editable_get_chars (GTK_EDITABLE(t), 0, -1);

    assert(NULL != (t = glade_xml_get_widget (xml, "propURItext")));
    pu = gtk_editable_get_chars (GTK_EDITABLE(t), 0, -1);

    assert(NULL != (t = glade_xml_get_widget (xml, "proppxuser")));
    px = gtk_editable_get_chars (GTK_EDITABLE(t), 0, -1);

    assert(NULL != (t = glade_xml_get_widget (xml, "propbrowser")));
    pb = gtk_editable_get_chars (GTK_EDITABLE(t), 0, -1);

    assert(NULL != (t = glade_xml_get_widget (xml, "propbasedir")));
    pd = gtk_editable_get_chars (GTK_EDITABLE(t), 0, -1);

    assert(NULL != (t = glade_xml_get_widget (xml, "propsremovecheck")));
    rem =  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t));    

    assert(NULL != (t = glade_xml_get_widget (xml, "propzips")));
    zip =  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t));    

    assert(NULL != (t = glade_xml_get_widget (xml, "proplocal")));
    loc =  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t));    

    assert(NULL != (t = glade_xml_get_widget (xml, "prophyph")));
    hyph =  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t));    

    check_for_change(&gblptr->proxy,   pp, &chg);
    check_for_change(&gblptr->uri,     pu, &chg);
    check_for_change(&gblptr->pxuser,  px, &chg);
    check_for_change(&gblptr->browser, pb, &chg);    

    /* This is sort of lame */
    if(*pd == '\0')
    {
        g_realloc(pd, 2);
        *pd = '.';
        *(pd+1) = 0;
    }

    check_for_change(&gblptr->basedir, pd, &chg);
    check_for_toggle (&gblptr->unlink_on_remove, rem, &chg);
    check_for_toggle (&gblptr->keepzips, zip, &chg);
    check_for_toggle (&gblptr->uselocal, loc, &chg);

    if(check_for_toggle (&gblptr->usehyph, hyph, &chg))
    {
        if(hyph)
        {
            gblptr->avail.total = loadhyphdicts(gblptr->avail.total);
        }
        else
        {
            int i,n = 0;
            for(i = gblptr->avail.total; i;)
            {
                i--;
                if(gblptr->avail.item[i].type == IS_HYPH)
                {
                    n++;
                    get_iter_from_n(model, i, &iter);
                    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
                }
            }
            gblptr->avail.total -= n;
        }
    }

    gtk_widget_hide(p);

    if(chg)
    {
        gblptr->flag |= UGLY_HACK;
        SaveConfigFile();
    }
}


void SaveConfigFile(void)
{
    char rcpath[PATH_MAX];
    char *p;
    FILE *fp;
    
    if(NULL != (p = getenv("HOME")))
    {
        buildpath(rcpath, p, ".ooodi", "ooodi.rc", NULL);
        if(NULL != (fp = fopen(rcpath,"w")))
        {
            fprintf(fp, "proxy = %s\n" , gblptr->proxy);
            if(gblptr->uri)
                fprintf(fp, "uri = %s\n", gblptr->uri);
            if(gblptr->pxuser && *gblptr->pxuser)
                fprintf(fp, "user = %s\n" , gblptr->pxuser);            
            if(gblptr->browser && *gblptr->browser)
            {
                fprintf(fp, "browser = %s\n", gblptr->browser);
            }
            if(gblptr->basedir && *gblptr->basedir && *gblptr->basedir != '.')
            {
                fprintf(fp, "basedir = %s\n", gblptr->basedir);
            }
            fprintf(fp, "unlink_on_remove = %d\n", gblptr->unlink_on_remove);
            fprintf(fp, "keep_zips = %d\n", gblptr->keepzips);
            fprintf(fp, "use_local = %d\n", gblptr->uselocal);
            fprintf(fp, "use_hyphen = %d\n", gblptr->usehyph);            
            fclose(fp);
        }
    }
}


void
on_modal_done                          (GtkButton       *button,
                                        gpointer         user_data)
{
    int *p;
    GtkWidget *w;
    assert(NULL != (w = glade_xml_get_widget (xml, "warning")));
    assert(NULL != (p = gtk_object_get_user_data (GTK_OBJECT(w))));
    gtk_widget_hide(w);
    *p = 1;
}

void
on_opt_ex_clicked                      (GtkObject       *object,
                                        gpointer         user_data)
{
    GtkWidget *w;
    FPDAT_t *u;
    assert(NULL != (w = glade_xml_get_widget (xml, "frontpage")));
    assert(NULL != (u = gtk_object_get_user_data (GTK_OBJECT(w))));
    u->act = ACT_EX;
}


void
on_opt_ed_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
    activate_prefs();
}


void
on_opt_dl_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *w;
    FPDAT_t *u;
    assert(NULL != (w= glade_xml_get_widget (xml, "frontpage")));
    assert(NULL != (u = gtk_object_get_user_data (GTK_OBJECT(w))));
    u->act = ACT_DL;
}


void
on_opt_fl_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *w;
    FPDAT_t *u;
    GtkWidget *fs;
    char title[PATH_MAX];
    char base[PATH_MAX];
    
    assert(NULL != (w = glade_xml_get_widget (xml, "frontpage")));
    assert(NULL != (u = gtk_object_get_user_data (GTK_OBJECT(w))));

    snprintf(title, sizeof(title), "%s - %s", _("Find File"), u->wanted);
    fs = create_filesel (title);
    
    gtk_object_set_user_data (GTK_OBJECT(fs), u);  
    make_local(base, "/");
    
    gtk_file_selection_set_filename (GTK_FILE_SELECTION(fs), base);
    gtk_file_selection_complete (GTK_FILE_SELECTION(fs), u->wanted);
    gtk_widget_show (fs);
}

void on_fs_ok_clicked (GtkObject *object, gpointer  user_data)
{
    GtkWidget *fs;
    FPDAT_t *u;
    char *fn;

    assert(NULL != (fs = glade_xml_get_widget (xml, "filesel")));
    assert(NULL != (u = gtk_object_get_user_data (GTK_OBJECT(fs))));
    fn = (char *)gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));

    switch(u->typ)
    {
        case AVAIL_TYPE:
        case HYPH_TYPE:
        {
            char *fnam = (u->typ == AVAIL_TYPE) ? AVAIL : HYPHLST;
            if(strstr(fn, fnam) && 0 == access(fn, R_OK))
            {
                u->fn = g_strdup(fn);
                u->act = ACT_FS;
            }
        }
        break;
        case BASE_TYPE:
        case USER_BASE:
            if(0 == access(fn, R_OK))
            {
                struct stat st;
                if(0 == stat(fn, &st))
                {
                    if(S_ISDIR(st.st_mode))
                    {
                        u->fn = g_malloc(PATH_MAX);
                        strcpy(u->fn, fn);
                        if(u->typ == BASE_TYPE)
                        {
                            strcat(u->fn, "/share/dict/ooo");
                        }
                        if(0 == access(u->fn, W_OK))
                        {
                            u->act = ACT_FS;
                        }
                        else
                        {
                            g_free(u->fn);
                        }
                    }
                }
            }
            break;
    }
    gtk_widget_hide(fs);
}


void on_fs_can_clicked (GtkObject *object, gpointer user_data)
{
    GtkWidget *fs;
    FPDAT_t *u;
    assert(NULL != (fs = glade_xml_get_widget (xml, "filesel")));
    assert(NULL != (u= gtk_object_get_user_data (GTK_OBJECT(fs))));
    u->act = ACT_CONT;
    gtk_widget_hide(fs);
}

void
on_urlbutton_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    if(gblptr->browser)
    {
        char cmd [PATH_MAX];
        snprintf(cmd, sizeof(cmd),"%s %s &", gblptr->browser, OOODI_URL);
        system(cmd);
    }
}


void
on_help_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    char cmd [PATH_MAX];
    snprintf(cmd, sizeof(cmd),"%s file://%s &", gblptr->browser, gblptr->help);
    system(cmd);
}



void
on_basebrowse_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *w;
    if((w = glade_xml_get_widget (xml, "props")) != NULL)
    {
        GtkWidget *fs;
        FPDAT_t u;
        u.act = ACT_NONE;
        u.typ = USER_BASE;

        fs = create_filesel (_("Select available file base directory"));
        gtk_object_set_user_data (GTK_OBJECT(fs), &u);  
        gtk_widget_set_sensitive (((GtkFileSelection *)fs)->file_list, 0);
        gtk_widget_show (fs);
        
        do
        {
            gtk_main_iteration();
        } while (u.act == ACT_NONE);

        if(u.act == ACT_FS)
        {
            GtkWidget *t;
            
            t = glade_xml_get_widget (xml, "propbasedir");
            gtk_entry_set_text(GTK_ENTRY(t), u.fn);
            g_free(u.fn);
        }
        
    }
}


void
on_repair_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    int i, ndict = 0;
    DICTLST_t *p = &gblptr->avail;
    char *q,*r, dict_file[PATH_MAX];
    
    q = stpcpy(dict_file, p->path);
    *q++ = '/';

    for(i = 0; i < p->total; i++)
    {
        if((p->item[i].flag & (SYSLOCMASK|USERLOCMASK)) == 0)
        {
            int j, nf = 0;
            int mx;

            mx = (p->item[i].type == IS_DICT) ? 2 : 1;
            
            for(j = 0; j < mx; j++)
            {
                r = stpcpy(q, p->item[i].spec);
                strcpy(r, afx[j]);
                if (access(dict_file,R_OK) == 0)
                {
                    nf++;
                }
            }
            
            if(nf == mx)
            {
                ndict++;
                p->item[i].flag |= DICT_AVAILABLE;
            }
        }
    }
    if(ndict)
    {
        update_dictfile(p, ADD_NEW, USER_LOCATION);
    }
}


void
on_copy_old_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gtk_widget_show(gblptr->copywin);    
}

static gint ffind(gconstpointer ldata, gconstpointer userdata)
{
    return(strncmp(ldata, userdata, strlen(userdata)));
}


static void copy_old_dicts(char *dir)
{
    FILE *fp;
    char *q, *r,file[PATH_MAX];
    int i;
    DICTLST_t *p = &gblptr->avail;
    
    q = stpcpy(file, dir);
    r = stpcpy(q, "/user/wordbook/");
    strcpy(r,  DICTLST);
    
    if(NULL != (fp = fopen(file, "r")))
    {
        char buf[128];
        int type;
        char *fbuf;            
        fbuf = g_malloc(4096);

        while(fgets(buf, sizeof(buf), fp))
        {
            char dtyp[5], *spec, *region;

            if(sscanf(buf, "%4s %*s %as %as", dtyp, &region, &spec) == 3)
            {
                if(strcmp(dtyp, DICT) == 0)
                {
                    type = IS_DICT;
                }
                else if(strcmp(dtyp, HYPH) == 0)
                {
                    type = IS_HYPH;
                }
                else
                {
                    type = -1;
                }
                
                if(-1 != type && -1 != (i = index_in_list(p, region, spec,
                                                          type)))
                {
                    FILE *ofp,*nfp;
                    char *s;
                    int j;
                    
                    for(j = 0; j < 2; j++)
                    {
                        s = stpcpy(r, spec);
                        strcpy(s, afx[j]);
                        
                        if(NULL != (ofp = fopen(file, "r")))
                        {
                            char npath[PATH_MAX];
                            int nb;
                            struct stat st;
                            
                            s = stpcpy(npath, p->path);
                            *s++ = '/';
                            s = stpcpy(s, spec);
                            strcpy(s, afx[j]);
                            
                            if(NULL != (nfp = fopen(npath, "w")))
                            {
                                struct utimbuf ut;
                                while((nb = fread(fbuf, 1, 4096, ofp)) > 0)
                                {
                                    fwrite(fbuf, 1, nb, nfp);
                                }
                                fclose(nfp);
                                fstat(fileno(ofp), &st);
                                ut.actime = ut.modtime = st.st_mtime;
                                utime(npath, &ut);
                            }
                            fclose(ofp);
                        }
                    }
                    p->item[i].flag |= DICT_AVAILABLE;
                }
                free(spec);
                free(region);
            }
        }
        g_free(fbuf);
        fclose(fp);
    }
    update_dictfile(p, ADD_NEW, USER_LOCATION);
}


void
on_copy_clicked                        (GtkObject       *object,
                                        gpointer         user_data)
{
    GtkWidget *w;
    assert(NULL != (w = glade_xml_get_widget (xml, "combo")));
    if(w)
    {
        char *string;
        string = (char *)gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(w)->entry));
        if(string && *string)
        {
            GList *ge;
            if(NULL != (ge = g_list_find_custom (gblptr->vers, string, ffind)))
            {
                char *dir;
                if(NULL != (dir = strchr(ge->data,'\t')))
                {
                    dir++;
                    copy_old_dicts(dir);
                }
            }
        }
        
    }
    gtk_widget_hide(gblptr->copywin);    
}

void
on_checkavail_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    char url[PATH_MAX];
    char name[PATH_MAX];
    int avst = OO_FILE_ERR;
    int hyst = OO_FILE_ERR;
    int i,stat = 0;
    GtkWidget *button;
    DLCB_t p;
    GtkTreeModel *model;
    GtkTreeIter iter;
    
    model = gtk_tree_view_get_model (GTK_TREE_VIEW (gblptr->avail.clist));

    p.flag = 0;
    assert(NULL != (button = glade_xml_get_widget (xml, "dlcancel")));
    gtk_object_set_user_data (GTK_OBJECT(button), &p);
    gtk_widget_set_sensitive (button, 1);

    set_busy_cursor(1); 

    make_local(name, AVAIL);    
    avst = OOFileStatus(AVAIL, name, 0);
    if(p.flag == 0)
    {
        if(OO_FILE_NEW ==  avst)
        {
            make_url(url, AVAIL);
            make_local(name, AVAIL);            
            if(0 == OODownload(url, name, ZIP_NOCHECK))
            {
                stat = 1;
            }
        }
    }

    if(p.flag == 0 && gblptr->usehyph)
    {
        make_local(name, HYPHLST);            
        hyst = OOFileStatus(HYPHLST, name, 0);

        if(p.flag == 0 && OO_FILE_NEW == hyst)
        {
            make_url(url, HYPHLST);
            if(0 == OODownload(url, name, ZIP_NOCHECK))
            {
                stat = 1;
            }
        }
    }
    
    if(p.flag == 0 && stat)
    {
        gtk_list_store_clear (GTK_LIST_STORE(model));
        for(i = 0; i <  gblptr->avail.total; i++)
        {
            g_free(gblptr->avail.item[i].lang);
            g_free(gblptr->avail.item[i].region);
            g_free(gblptr->avail.item[i].spec);
            g_free(gblptr->avail.item[i].desc);
            g_free(gblptr->avail.item[i].file);
        }
        load_the_lists();
    }
    
    for(i = 0; i <  gblptr->avail.total; i++)
    {
        char *pth;
        int new = 0;
        char *q,*r, *zip;

            /* For hyphs mainly, don't check duplicate
             * names -- it's tedious on a 56K modem
             */
        
        if(i)
        {
            if(strcmp(gblptr->avail.item[i-1].spec,
                      gblptr->avail.item[i].spec) == 0)
                continue;
        }
        
        if(gblptr->avail.item[i].flag & ( USERLOCMASK|SYSLOCMASK))
        {
            pth = (gblptr->avail.item[i].flag & USERLOCMASK) ?
                gblptr->avail.path : gblptr->avail.path_shr;
            
            q = stpcpy(name, gblptr->basedir);
            *q++ = '/';
            r = stpcpy(q, gblptr->avail.item[i].spec);
            strcat(r,".zip");

            new = OOFileStatus(q, name, 0);
            if(p.flag)
                break;
            
            if(new < 0)
            {
                zip = g_strdup(q);
                q = stpcpy(name, pth);
                *q++ = '/';
                r = stpcpy(q, gblptr->avail.item[i].spec);
                strcat(r,".dic");
                new = OOFileStatus(zip, name, DICTDATEOFFSET);
                g_free(zip);
                if(p.flag)
                    break;
            }
            
            if(new == OO_FILE_NEW)
            {
                char *txt;

                get_iter_from_n(model, i, &iter);
                gtk_tree_model_get(model, &iter, TYPE_COL, &txt, -1);
                
                {
                    char *t,*txt1;
                        /* Only add * if not there already */
                    if(NULL == (t = strchr(txt, '*')))
                    {
                        GValue gv = {0};
                        g_value_init (&gv, G_TYPE_STRING);                        
                        txt1 = g_malloc(strlen(txt) + 2);
                        t = stpcpy(txt1, txt);
                        *t++ = '*';
                        *t = 0;

                        g_value_set_string(&gv, txt1);
                        gtk_list_store_set_value (GTK_LIST_STORE(model), &iter, TYPE_COL, &gv);
                        g_free(txt);
                        g_free(txt1);
                    }
                }
            }
        }
    }
    gtk_widget_set_sensitive (button, 0);
    set_busy_cursor(0);
}

static void doselection (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, 
                         gpointer data)
{
    if (gtk_tree_selection_iter_is_selected (data, iter))
    {
        DICTLST_t *p = &gblptr->avail;
        int row;
        char *str;
        str = gtk_tree_path_to_string(path);
        row = atoi(str);
        
        p->item[row].flag |= DICT_SELECTED;
        
        if(p->location & USERLOCMASK)
        {
            p->iselected++;
            
            if(p->item[row].flag & DICT_INSTALLED)
            {
                p->rselected++;
            }
        }
        
        if(p->location & SYSLOCMASK)
        {
            p->s_iselected++;
            
            if(p->item[row].flag & DICT_INSTALLED_S)
            {
                p->s_rselected++;
            }
        }
    }
}

void tree_select_cb (GtkTreeSelection *s, gpointer data)
{
    DICTLST_t *p = &gblptr->avail;
    int i;
    
    p->iselected = p->rselected = p->s_iselected = p->s_rselected = 0;
    
    for(i = 0; i < p->total; i++)
    {
        p->item[i].flag &= ~DICT_SELECTED;
    }

    gtk_tree_selection_selected_foreach (s, doselection, s);

    gtk_widget_set_sensitive (p->install, p->iselected);
    gtk_widget_set_sensitive (p->remove, p->rselected);
    gtk_widget_set_sensitive (p->install_s, p->s_iselected);
    gtk_widget_set_sensitive (p->remove_s, p->s_rselected);
}



