/*
 *  OOo Dictionary Installer (c) Jonathan Hudson 2002.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <curl/curl.h>
#include <utime.h>
#include "ooodi.h"
#include <assert.h>
static int curl_ini;

static int writecb(void *buf, size_t size, size_t nmemb, DLCB_t *ptr)
{
    int res = -1;
    
    res = fwrite(buf, size, nmemb, ptr->fp);
    if(ptr->zflag == ZIP_CHECK)
    {
        if(memcmp(buf, ZIPSIG, sizeof(ZIPSIG)-1) != 0)
        {
            ptr->flag = ZIPERR;
        }
        ptr->zflag = ZIP_CHECKED;
    }

    return res;
}

#if LIBCURL_VERSION_NUM < 0x070905
typedef size_t CBSIZE_ARG_t;
#else
typedef double CBSIZE_ARG_t;
#endif


static int cbfunc(DLCB_t *p, CBSIZE_ARG_t dltotal, CBSIZE_ARG_t  dlnow,
                  CBSIZE_ARG_t ultotal, CBSIZE_ARG_t ulnow)
{
    gfloat pct;
    
    while (gtk_events_pending())
        gtk_main_iteration();

    if(0 == p->flag)
    {
        pct = 100.0*dlnow/dltotal;
        gtk_progress_set_value (GTK_PROGRESS( ((DLCB_t *) p)->widget), pct);
    }

    while (gtk_events_pending())
        gtk_main_iteration();

    return p->flag;
}

static void set_curl_proxy(CURL *c)
{
    curl_easy_setopt(c, CURLOPT_PROXY, gblptr->proxy);
    if(gblptr->pxuser && *gblptr->pxuser)
    {
        char txt[256];                        
        if(NULL == gblptr->pxpass)
        {
            int p = 0;
            GtkWidget *pw,*t;
            assert(NULL != (pw = glade_xml_get_widget (xml, "passdlg")));
            gtk_object_set_user_data (GTK_OBJECT(pw), &p);  
            snprintf(txt,sizeof(txt),"%s\nPassword for %s",
                     gblptr->proxy, gblptr->pxuser);
            assert(NULL != (t = glade_xml_get_widget (xml, "passlabel")));
            gtk_label_set_text(GTK_LABEL(t), txt);
            gtk_widget_show(pw);
            while(1)
            {
                gtk_main_iteration();
                if(p) break;
            }
            assert(NULL != (t = glade_xml_get_widget (xml, "passentry")));
            gblptr->pxpass =
                gtk_editable_get_chars (GTK_EDITABLE(t), 0, -1);
        }
        if(gblptr->pxpass && *gblptr->pxpass)
        {
            snprintf(txt,sizeof(txt),"%s:%s",
                     gblptr->pxuser,gblptr->pxpass);
            curl_easy_setopt(c, CURLOPT_PROXYUSERPWD, txt);
        }
    }
}

int OOFileStatus (char *file, char *path, unsigned int offset)
{
    time_t mtime = -1;
    int res = -1;
    struct stat st;

    if(0 == stat(path, &st))
    {
        if(0 == (res = OORemoteStatus(file, &mtime)))
        {
            if(mtime != -1 )
            {
                time_t ftime;
                ftime = (st.st_mtime != -1) ? st.st_mtime : st.st_ctime;
                if(mtime > (ftime + offset))
                {
                    res = OO_FILE_NEW;
                }
                else
                {
                    res = OO_FILE_CURR;
                }
            }
            else
            {
                res = OO_FILE_UNKNOWN;
            }
        }
        else
        {
            res = OO_FILE_ERR;            
        }
    }
    else
    {
        res = OO_FILE_ERR;
    }
    return res;
}


int OORemoteStatus (char *file, time_t *ftime)
{
    time_t mtime = -1;
    char url[PATH_MAX];
    CURL *c ;
    int res = -1;
    
    while (gtk_events_pending())
        gtk_main_iteration();

    if((c = curl_easy_init()) != NULL)
    {
        char *p;
        
        p = stpcpy(url, gblptr->uri);
        *p++ = '/';
        strcpy(p, file);        

        curl_easy_setopt(c, CURLOPT_FILETIME, 1);
        curl_easy_setopt(c, CURLOPT_NOBODY, 1);
        curl_easy_setopt(c, CURLOPT_URL, url);
        if(gblptr->proxy && *gblptr->proxy)
        {
            set_curl_proxy(c);
        }
        while (gtk_events_pending())
            gtk_main_iteration();
        res = curl_easy_perform(c);
        while (gtk_events_pending())
            gtk_main_iteration();
        curl_easy_getinfo(c, CURLINFO_FILETIME, &mtime);
        curl_easy_cleanup(c);
        *ftime = mtime;
    }
    return res;
}


int OODownload(char *url, char *file, int flag)
{
    GtkWidget *title, *button, *text;
    GtkTextIter iter;
    
    CURL *c;
    DLCB_t p;
    int res = -2;

    p.flag = 0;
    p.zflag = flag;
    
    if(!curl_ini)
    {
        g_atexit(curl_global_cleanup);
    }

    assert(NULL != (title = glade_xml_get_widget (xml, "dllabel")));
    assert(NULL != (button = glade_xml_get_widget (xml, "dlcancel")));
    assert(NULL != (text = glade_xml_get_widget (xml, "errtext")));
    assert(NULL != (p.widget = glade_xml_get_widget (xml, "progressbar1")));
    
    {
        char errbuf[CURL_ERROR_SIZE];
        *errbuf = '\0';

        if((c = curl_easy_init()) != NULL)
        {
            p.fp = fopen(file,"w");
            if(NULL != p.fp)
            {
                time_t mtime = 0;
                
                if(title)
                {
                    gtk_label_set_text(GTK_LABEL(title), file);
                }
                gtk_object_set_user_data (GTK_OBJECT(button), &p);
                gtk_widget_set_sensitive (button, 1);
                gtk_progress_set_show_text(GTK_PROGRESS(p.widget), 1);
                gtk_widget_set_sensitive (p.widget, 1);
                gtk_widget_set_sensitive (gblptr->avail.install, 0);

                curl_easy_setopt(c, CURLOPT_URL, url);
                curl_easy_setopt(c, CURLOPT_FILE, &p);
                curl_easy_setopt(c, CURLOPT_WRITEFUNCTION, writecb);
                curl_easy_setopt(c, CURLOPT_PROGRESSFUNCTION, cbfunc);
                curl_easy_setopt(c, CURLOPT_PROGRESSDATA, &p);
                curl_easy_setopt(c, CURLOPT_NOPROGRESS, 0);
                curl_easy_setopt(c, CURLOPT_ERRORBUFFER, errbuf);
                curl_easy_setopt(c, CURLOPT_FILETIME, 1);

                if(gblptr->proxy && *gblptr->proxy)
                {
                    set_curl_proxy(c);
                }
                
                gtk_progress_set_value (GTK_PROGRESS(p.widget), 0.0);
                gtk_object_set_user_data (GTK_OBJECT(p.widget), &p);
                    /* Give GTK a look in */
                while (gtk_events_pending())
                    gtk_main_iteration();
                
                res = curl_easy_perform(c);
                
                while (gtk_events_pending())
                    gtk_main_iteration();
                
                curl_easy_getinfo(c, CURLINFO_FILETIME, &mtime);
                curl_easy_cleanup(c);
                fclose(p.fp);
                if(mtime)
                {
                    struct utimbuf ut;
                    ut.actime = ut.modtime = mtime;
                    utime(file, &ut);
                }

                if(res || p.flag == ZIPERR)
                {
                    char txtbuf[512];
                    if(p.flag == ZIPERR)
                    {
                        
                        char *newf = g_malloc(strlen(file) + 16);
                        if(newf)
                        {
                            strcat(stpcpy(newf, file),".failed");
                            rename(file, newf);
                            g_free(newf);
                        }
                        snprintf(txtbuf, sizeof(txtbuf),
                                 _("%s: ZIP file error (see %s.failed)\n"),
                                 file, file);
                        res = -1;
                    }
                    else
                    {
                        unlink(file);
                        snprintf(txtbuf, sizeof(txtbuf),
                                 "%s: %s\n", file, errbuf);
                    }

                    
                    gtk_text_buffer_get_end_iter (gtk_text_view_get_buffer (GTK_TEXT_VIEW(text)),
                                                  &iter);

                     gtk_text_buffer_insert  (gtk_text_view_get_buffer (GTK_TEXT_VIEW(text)),
                                             &iter, txtbuf, -1);
                }
                gtk_label_set_text(GTK_LABEL(title), "");
            }
            else
            {
                DisplayWarning(gblptr->warn,file,
                           _("Cannot open file to save download"));
            }
        }
        gtk_widget_set_sensitive (button, 0);
        gtk_widget_set_sensitive (gblptr->avail.install,
                                  gblptr->avail.iselected);
        gtk_progress_set_value (GTK_PROGRESS(p.widget), 0.0);
        gtk_progress_set_show_text(GTK_PROGRESS(p.widget), 0);
        gtk_widget_set_sensitive (p.widget, 0);

    }
    return res;
}

