/*
 *  OOo Dictionary Installer (c) Jonathan Hudson 2002.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <popt.h>
#include <assert.h>

/* Support older versions of popt */
#ifndef POPT_TABLEEND
# define POPT_TABLEEND { NULL, '\0', 0, 0, 0, NULL, NULL }
#endif

#include "ooodi.h"
#include <pixmaps/pix.h>

GladeXML *xml;
OOODL_GBL_t *gblptr;
static GtkListStore *store;
static GtkTreeIter iter;    

void DisplayWarning(GtkWidget *w, char *title, char *label)
{
    GtkWidget *t;
    static int p = 0;
    char *pul,*ul;

    gtk_window_set_title (GTK_WINDOW(w), title);
    gtk_object_set_user_data (GTK_OBJECT(w), &p);  
    assert(NULL != (t = glade_xml_get_widget (xml, "warntitle")));
    ul = g_strdup(title);
    for(pul = ul; *pul; *pul++ = '_')
        ; /* null loop body */

    gtk_label_set_text(GTK_LABEL(t), title);
    gtk_label_set_pattern(GTK_LABEL(t), ul);
    
    assert(NULL != (t = glade_xml_get_widget (xml, "warnlabel")));
    gtk_label_set_text(GTK_LABEL(t), label);
    gtk_widget_show(w);
    g_free(ul);
    
    while (1)
    {
        gtk_main_iteration();
        if(p) break;
    }
}


/*
 * Establish various window parameters, in particular names we need
 * for later lookups
 */

static void setup_windows()
{
    GtkWidget *w;

    
    gtk_widget_set_usize (gblptr->window, -1, 400);

    assert(NULL != (w = glade_xml_get_widget (xml, "oootitle")));
    gtk_widget_set_name(w, "oootitle");

    assert(NULL != (w = glade_xml_get_widget (xml, "selinstr")));
    gtk_widget_set_name(w, "selinst");

    assert(NULL != (w = glade_xml_get_widget (xml, "aboutl1")));
    gtk_widget_set_name(w, "aboutl1");
    
    assert(NULL != (w = glade_xml_get_widget (xml, "installbutton")));
    gblptr->avail.install = w;
    gtk_widget_set_sensitive (w, 0);
    gtk_object_set_user_data (GTK_OBJECT(w), (void *)USER_LOCATION);  
    

    assert(NULL != (w = glade_xml_get_widget (xml, "removebutton")));
    gblptr->avail.remove = w;
    gtk_widget_set_sensitive (w, 0);
    gtk_object_set_user_data (GTK_OBJECT(w), (void *)USER_LOCATION);  
    
    assert(NULL != (w = glade_xml_get_widget (xml, "installshare")));
    gtk_widget_set_sensitive (w, 0);
    gblptr->avail.install_s = w;
    gtk_object_set_user_data (GTK_OBJECT(w), (void *)SYSTEM_LOCATION);  
    

    assert(NULL != (w = glade_xml_get_widget (xml, "removeshare")));
    gtk_widget_set_sensitive (w, 0);
    gblptr->avail.remove_s = w;
    gtk_object_set_user_data (GTK_OBJECT(w), (void *)SYSTEM_LOCATION);  
    
    
    assert(NULL != (w = glade_xml_get_widget (xml, "clist1")));
    gblptr->avail.clist = w;
    

    assert(NULL != (w = glade_xml_get_widget (xml, "dlcancel")));
    gtk_widget_set_sensitive (w, 0);

    gtk_signal_connect_object(GTK_OBJECT(gblptr->window), "button_press_event",
                              GTK_SIGNAL_FUNC (on_ooo_window_button_press_event),
                              GTK_OBJECT(gblptr->popmenu));

    assert(NULL != (w = glade_xml_get_widget (xml, "Help")));
    gtk_menu_item_right_justify(GTK_MENU_ITEM (w));
    

    assert(NULL != (w = glade_xml_get_widget (xml, "errtext")));
    gtk_text_view_set_buffer (GTK_TEXT_VIEW(w), gtk_text_buffer_new (NULL));
    
    
    assert(NULL != (w = glade_xml_get_widget (xml, "help2")));
    
    if(gblptr->browser)
    {
        char **hp, *hpaths [] ={PACKAGE_DATA_DIR,
                                "/usr/local/share/OOodi2",
                                "/usr/share/OOodi2", NULL};
        char tmp[PATH_MAX];
        
        for(hp = hpaths; *hp; hp++)
        {
            buildpath(tmp, *hp, OOODI_HELP, NULL);
            if(0 == access(tmp, R_OK))
            {
                gblptr->help = g_strdup(tmp);
                break;
            }
        }
        if(!gblptr->help)
        {
            gtk_widget_set_sensitive (w, 0);
        }
    }

    setwinicon (gblptr->window);
    setwinicon (gblptr->props);
    setwinicon (gblptr->about);      
}

static void seturlcolour()
{
    GtkWidget *w;
    
    assert(NULL != (w = glade_xml_get_widget (xml, "urlbutton")));
    {
        GdkColor uri_colour[] = {{0, 0, 0, 0xffff}, {0, 0xffff, 0, 0}};
        GtkStyle *style;
        GdkColormap *cmap;
        gboolean success[2];

        cmap = gdk_window_get_colormap(gblptr->window->window);
        gdk_colormap_alloc_colors(cmap, uri_colour, 2, FALSE, TRUE, success);
        if (success[0] == TRUE && success[1] == TRUE) {
            gtk_widget_ensure_style(GTK_BIN(w)->child);
            style = gtk_style_copy
                (gtk_widget_get_style(GTK_BIN(w)->child));
            style->fg[GTK_STATE_NORMAL]   = uri_colour[0];
            style->fg[GTK_STATE_ACTIVE]   = uri_colour[1];
            style->fg[GTK_STATE_PRELIGHT] = uri_colour[0];
            gtk_widget_set_style(GTK_BIN(w)->child, style);
        }
    }
}

static void setbgcolour()
{
    GtkWidget *w;
    GdkColor white = { 0, 0xffff, 0xffff, 0xffff };
    GtkRcStyle *rc_style = gtk_rc_style_new ();
    rc_style->bg[GTK_STATE_NORMAL] = white;
    rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_BG;

    assert(NULL != (w = glade_xml_get_widget (xml, "eventbox1")));
    gtk_widget_modify_style (w, rc_style);
    
    assert(NULL != (w = glade_xml_get_widget (xml, "eventbox2")));
    gtk_widget_modify_style (w, rc_style);
    gtk_rc_style_unref (rc_style);
}


static void read_dict_file (char *filename, short flag)
{
    FILE *fp;
    
    if(NULL != (fp = fopen(filename, "r")))
    {
        char dicts[128];
        char *region,*spec,*lang;
        char dtyp[5];
        int n;
        short type;
        
        while(fgets(dicts, sizeof(dicts), fp))
        {
            if(4 == sscanf(dicts, "%4s %as %as %as", dtyp, &lang, &region, &spec))
            {
                if(strcmp(dtyp, DICT) == 0)
                {
                    type = IS_DICT;
                }
                else if(strcmp(dtyp, HYPH) == 0)
                {
                    type = IS_HYPH;
                }
                else
                    continue;
                    
                for(n = 0; n < gblptr->nd; n++)
                {
                    if(0 == strcmp(gblptr->dlst[n].name, spec) &&
                       0 == strcmp(gblptr->dlst[n].region, region) &&
                       gblptr->dlst[n].type == type)
                    {
                        break;
                    }
                }

                if(n < gblptr->nd) /* Entry exists is another area */
                {
                    gblptr->dlst[n].location |= (1 << flag);
                    free(spec);
                    free(region);
                }
                else
                {
                    if(gblptr->nd == gblptr->ndmax)
                    {
                        gblptr->dlst = g_realloc(gblptr->dlst, 32*sizeof(DICTNAMES));
                        memset(gblptr->dlst+gblptr->ndmax, '\0', 32*sizeof(DICTNAMES));
                        gblptr->ndmax += 32;
                    }
                    gblptr->dlst[gblptr->nd].name = spec;
                    gblptr->dlst[gblptr->nd].region = region;
                    gblptr->dlst[gblptr->nd].location |= (1 << flag);
                    gblptr->dlst[gblptr->nd].type = type;
                    gblptr->nd++;
                }
            }
        }
        fclose(fp);
    }
}

static void browse_and_share()
{
    char link[PATH_MAX];
    char *p;
    
    FPDAT_t u;

    if(gblptr->avail.path_shr == NULL)
    {
        GtkWidget *fs;
        u.act = ACT_NONE;
        u.typ = BASE_TYPE;

        fs = create_filesel (_("Select OO.o installation directory"));
        gtk_object_set_user_data (GTK_OBJECT(fs), &u);  
        gtk_widget_set_sensitive (((GtkFileSelection *)fs)->file_list, 0);
        gtk_widget_show (fs);
        
        do
        {
            gtk_main_iteration();
        } while (u.act == ACT_NONE);

        if(u.act == ACT_FS)
        {
            gblptr->avail.path_shr = u.fn;
        }
        else
        {
            gtk_exit(0);
        }
    }

    if(gblptr->avail.path_shr != NULL)
    {
        gblptr->avail.location = SYSLOCMASK;
        p = stpcpy(link, gblptr->avail.path_shr);
        strcat(p, "/dictionary.lst");
        read_dict_file(link, SYSTEM_LOCATION);
    }
}

static void fix_buttons()
{
    GtkWidget *w;

    if(!(gblptr->avail.location & USERLOCMASK))
    {
        assert(NULL != (w = glade_xml_get_widget (xml, "installbutton")));
        gtk_widget_hide (w);
        
        assert(NULL != (w = glade_xml_get_widget (xml, "removebutton")));
        gtk_widget_hide (w);
    }

    if(!(gblptr->avail.location & SYSLOCMASK))
    {
        assert(NULL != (w = glade_xml_get_widget (xml, "installshare")));
        gtk_widget_hide (w);

        assert(NULL != (w = glade_xml_get_widget (xml, "removeshare")));
        gtk_widget_hide (w);
    }
}


static void im_free(gpointer data, gpointer user_data)
{
    user_data = user_data;
    g_free(data);
}

/*
 * Find the user path (from the last OOo in .sversionrc).
 * Find the installed dictionaries
 */

static void parse_user_dicts()
{
    char path[PATH_MAX], link[PATH_MAX];
    char *p, *q, *s, *vers;
    FILE *fp;
    int fail = 0;
    
    /* This is a bit ugly, but it we enter with the path to the shared
     * area set, then (a) we are root and (b) we passed the path on the
     * command line and (c) we're only going to install shared dictionaries
     */
         
    if(gblptr->avail.path_shr)
    {
        browse_and_share();
    }
    else
    {
        p = getenv("HOME");

        if(p == NULL)
        {
            if(0 == geteuid())
            {
                browse_and_share();
            }
            else
            {
                fail = 1;
            }
        }
        else
        {
            q = stpcpy(path, p);
            strcpy (q, "/.sversionrc");
            fp = fopen(path,"r");

            if(NULL != fp)
            {
                char buf[PATH_MAX];
                GList *glist = NULL;
                
                s = vers = NULL;
                
                while(fgets(buf, sizeof(buf), fp))
                {
                    char *xvers = NULL;
                    char *xs = NULL;
                                       
                    if( 2 == sscanf(buf,
                                 "OpenOffice.org%a[ 0-9.]=file://%a[^\r\n]",
                                    &xvers, &xs))
                    {
                        if(vers)
                        {
                            char entry[128];
                            snprintf(entry, sizeof(entry), "%s\t%s", vers, s);
                            gblptr->vers = g_list_append(gblptr->vers, g_strdup(entry));
                            glist = g_list_append(glist, vers);                            
                        }
                        if(s)
                        {
                            g_free(s);
                        }
                        s = xs;
                        vers = xvers;
                    }
                }
                fclose(fp);

                if(glist)
                {
                    GtkWidget *combo;
                    assert(NULL != (gblptr->copywin = glade_xml_get_widget (xml, "copywin")));
                    assert(NULL != (combo = glade_xml_get_widget (xml, "combo")));
                    gtk_combo_set_popdown_strings( GTK_COMBO(combo), glist);
                    g_list_foreach(glist, im_free, NULL);
                    g_list_free (glist);
                    
                }
                else
                {
                    GtkWidget *w;
                    assert(NULL != (w = glade_xml_get_widget (xml, "copyold")));
                    gtk_widget_set_sensitive (w, 0);        
                }
                
                if(s)
                {
                    char follow[PATH_MAX];                  
                    char *gptr;
                    int nl;

                    s = g_realloc(s, PATH_MAX);
                    if(access(s, W_OK) == 0)
                    {
                        gptr = stpcpy(follow, s);
                        strcpy(gptr, "/soffice");
                        nl = readlink(follow, link, sizeof(link));
                        if(nl != -1)
                        {
                            char *pp;
                            pp = strstr(link, "/program");
                            if(pp)
                            {
                                strcpy(pp,"/share/dict/ooo");
                                if(gblptr->avail.path_shr)
                                {
                                    g_free(gblptr->avail.path_shr);
                                }
                                gblptr->avail.path_shr = g_strdup(link);
                                if(access(link, W_OK) == 0)
                                {
                                    gblptr->avail.location |= SYSLOCMASK;
                                }
                            }
                        }
                        strcat(s, "/user/wordbook");
                        gblptr->avail.path = g_strdup(s);
                        strcat(s, "/dictionary.lst");
                        gblptr->avail.location |= USERLOCMASK;
                        read_dict_file(s, USER_LOCATION);
                        if(gblptr->avail.path_shr)
                        {
                            strcat(link, "/dictionary.lst");
                            read_dict_file(link, SYSTEM_LOCATION);
                        }
                        g_free(s);
                    }
                    else
                    {
                        fail = 1;
                    }
                }
                else
                {
                    fail = 1;
                }
                
            }
            else if(0 == geteuid())
            {
                browse_and_share();
            }
            else
            {
                fail = 1;
            }
        }
    }
    if(fail)
    {
        DisplayWarning(gblptr->warn, _("OpenOffice"),
                       _("Cannot find user installation!"));
        gtk_exit(0);
    }
}

int read_available_file(char *filename, short type, int n)
{
    FILE *fp;
    if( (fp = fopen(filename,"r")) != NULL)
    {
        char buf[256];
        GIConv icv;

        icv = g_iconv_open ("UTF-8", "ISO-8859-1");
        
        while(fgets(buf, sizeof(buf), fp) != NULL)
        {
            int j;
            char *flds[5] = {NULL};
            
            memset(flds, sizeof(flds), 0);

            if(5 == sscanf(buf, "%a[^,],%a[^,],%a[^,],%a[^,],%a[^\r\n]",
                           flds, flds+1, flds+2,flds+3,flds+4))
            {
                if(icv != (GIConv)-1)
                {
                    gsize ibs, obs;
                    gchar *x;
                    GError *err;
                    
                    x = g_convert_with_iconv(flds[3], strlen(flds[3]), icv,
                                  &ibs, &obs, &err);
                    
                    if(x == NULL)
                    {
                        fprintf(stderr, "Failed for %s (%s)\n", flds[3],
                                err->message);
                        gblptr->avail.item[n].desc = flds[3];
                        g_error_free (err);
                    }
                    else
                    {
                        gblptr->avail.item[n].desc = x;
                        free(flds[3]);
                    }
                }
                else
                {
                    gblptr->avail.item[n].desc = flds[3];
                }
                
                gblptr->avail.item[n].lang = flds[0];
                gblptr->avail.item[n].region = flds[1];                
                gblptr->avail.item[n].spec = flds[2];
                gblptr->avail.item[n].file = flds[4];
                gblptr->avail.item[n].type = type;
                
                if(type == IS_DICT)
                {
                    if(0 != strncmp(gblptr->avail.item[n].spec,
                                    gblptr->avail.item[n].file,
                                    strlen(gblptr->avail.item[n].spec)))
                    {
                        gblptr->avail.item[n].flag |= DICT_SPECIAL_LINK;
                    }
                }
                
                for(j = 0; j < gblptr->nd; j++)
                {
                    if(0 == strcmp(gblptr->avail.item[n].spec,
                                   gblptr->dlst[j].name) &&
                       0 == strcmp(gblptr->avail.item[n].region,
                                   gblptr->dlst[j].region) &&
                       gblptr->dlst[j].type == type)
                    {
                        if(gblptr->dlst[j].location & (1 << USER_LOCATION))
                        {
                            gblptr->avail.item[n].flag |= DICT_INSTALLED;
                        }
                        if(gblptr->dlst[j].location & (1 << SYSTEM_LOCATION))
                        {
                            gblptr->avail.item[n].flag |= DICT_INSTALLED_S;
                        }
                        break;
                    }
                }

                gtk_list_store_append(store, &iter);    
                gtk_list_store_set(store, &iter,
                                   USER_COL, gblptr->avail.item[n].flag & DICT_INSTALLED,
                                   SYS_COL, gblptr->avail.item[n].flag & DICT_INSTALLED_S,
                                   NAME_COL, gblptr->avail.item[n].desc,
                                   TYPE_COL, (type == IS_DICT) ?  _("Dict") :  _("Hyph"),
                                   -1);
                n++;
            }
            else
            {
                for(j = 0; j < 5; j++)
                {
                    if(flds[j])
                        free(flds[j]);
                }
            }
        }
        if(icv != (GIConv)(-1))
            g_iconv_close(icv);
        fclose(fp);
    }
    
    return n;
}


char *ask_user_for_file(char *filename, char *ftype)
{
    char name[PATH_MAX];
    FPDAT_t u;
    GtkWidget *w,*v;
    char *old = NULL;
    static char *dtyp[] = {"SPELLING", "HYPHENATION"};
    char *dptr;
    

    u.act = ACT_NONE;
    u.typ = AVAIL_TYPE;
    u.wanted = ftype;
            
    assert(NULL != (w= glade_xml_get_widget (xml, "frontpage")));
    gtk_object_set_user_data (GTK_OBJECT(w), &u);
    setwinicon (w);
    snprintf(name, sizeof(name),"%s : %s", _("File not found"), ftype);
    gtk_window_set_title (GTK_WINDOW(w), name);
    
    assert(NULL != (v = glade_xml_get_widget (xml, "opt_ex")));
    
    if(strcmp(ftype, AVAIL) != 0)
    {
        dptr = dtyp[1];
        u.typ = HYPH_TYPE;
        gtk_object_set(GTK_OBJECT(v), "label",
                       _("Don't bother with hyphenation"), NULL);
}
    else
    {
        dptr = dtyp[0];
    }
    

    snprintf(name, sizeof(name),
             _("You currently don't appear to have a list of %s dictionaries on your "
               "computer. This is quite normal.\n\n"
               "Please select an option from the list below.\n\n"
               "* If your have a local file, you can modify the 'Dictionary' entry in your "
               "settings to locate a local file named %s.\n\n"
               "* If you are behind a proxy, you may need to add or modify proxy settings "
               "to be able to download the file from the internet.\n\n"
               
               "Once you are satisfied with your settings, you can download the "
               "required file."),
             dptr, ftype);

    assert(NULL != (v = glade_xml_get_widget (xml, "fptitle")));
    gtk_object_set(GTK_OBJECT(v), "label", name, NULL);

    old = g_strdup(gblptr->basedir);
    gtk_widget_show(w);
    
    do
    {
        gtk_main_iteration();
        if(gblptr->flag & UGLY_HACK)
        {
            gblptr->flag &= ~UGLY_HACK;
            if(0 != strcmp(old, gblptr->basedir))
            {
                buildpath(filename, gblptr->basedir, ftype, NULL);
                if(access(filename,R_OK) == 0)
                {
                    u.act = ACT_ED;
                }
            }
        }
    } while (u.act == ACT_NONE || u.act == ACT_CONT);
    
    gtk_widget_hide(w);    
    
    switch(u.act)
    {
        case ACT_DL: /* Download */
            buildpath(name, gblptr->uri, ftype, NULL);
            OODownload(name, filename, ZIP_NOCHECK);
            break;
        case ACT_FS: /* Get local copy */
            filename = u.fn;
            break;
        case ACT_ED:
            break;
        default:
            filename = NULL;
            if(strcmp(ftype, AVAIL) == 0)            
                gtk_exit(0);
            break;
    }

    if(old)
    {
        g_free(old);
    }
    return filename;
}


int loadhyphdicts(int nt)
{
    char *hf, hfile[PATH_MAX];
    
    buildpath (hfile, gblptr->basedir, HYPHLST, NULL);
    if(access(hfile, R_OK) != 0)
    {
        hf = ask_user_for_file(hfile, HYPHLST);
    }
    else hf = hfile;
    if(hf)
    {
        nt = read_available_file(hf, IS_HYPH, nt);
    }
    return nt;
}

void load_the_lists(void)
{
    int nt = 0;
    char availfn[PATH_MAX];
    char *filename;

    filename = buildpath (availfn, gblptr->basedir, AVAIL, NULL);
    filename = availfn;

    while(nt == 0)
    {
        if(gblptr->flag)
        {
            filename = ask_user_for_file(filename, AVAIL);
        }

        nt = read_available_file (filename, IS_DICT, nt);
        
        if(nt == 0)
        {
            gblptr->flag = 1;
            DisplayWarning(gblptr->warn, _("Available File"),
                           _("Cannot load file available.lst\n"
                             "Please check that this file exists\n"
                             "and contains dictionary entries\n"));
            break;
        }

        if(gblptr->usehyph)
        {
            nt = loadhyphdicts(nt);
        }
    }
    gblptr->avail.total = nt;
}


/*
 * Load the available.lst file
 */

static gint loaddictlist(gpointer ip)
{
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    GtkTreeSelection *select;
    
    store = gtk_list_store_new (NO_COLS,
                                G_TYPE_BOOLEAN,
                                G_TYPE_BOOLEAN,
                                G_TYPE_STRING,
                                G_TYPE_STRING);

    load_the_lists();
    
    renderer = gtk_cell_renderer_toggle_new ();
    column = gtk_tree_view_column_new_with_attributes (_("User"),
                                                       renderer,
                                                       "active", USER_COL,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (gblptr->avail.clist),
                                 column);
    column = gtk_tree_view_column_new_with_attributes (_("Shared"),
                                                       renderer,
                                                       "active", SYS_COL,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (gblptr->avail.clist),
                                 column);
    
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes (_("Dictionary"),
                                                       renderer,
                                                       "text", NAME_COL,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (gblptr->avail.clist),
                                 column);
    
    column = gtk_tree_view_column_new_with_attributes (_("Type"),
                                                       renderer,
                                                       "text", TYPE_COL,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (gblptr->avail.clist),
                                 column);

    gtk_tree_view_set_model(GTK_TREE_VIEW(gblptr->avail.clist), GTK_TREE_MODEL(store));
    g_object_unref(G_OBJECT(store));

    select = gtk_tree_view_get_selection(GTK_TREE_VIEW (gblptr->avail.clist));
    gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
    g_signal_connect(G_OBJECT(select), "changed", G_CALLBACK(tree_select_cb), NULL);
    gtk_widget_show(gblptr->avail.clist);    
    return 0;
}

/*
 * The basic GTKRC file. Sets some sizes for titles
 */

static char *baserc = "style \"title_label\"\n" 
"{\n" 
" font_name = \"Arial Bold 24\"\n"
"}\n"
"style \"subtitle_label\"\n"
"{\n"
" font_name = \"Arial 18\"\n"
"}\n"
"widget \"*oootitle\" style \"title_label\"\n"
"widget \"*aboutl1\" style \"subtitle_label\"\n";

/*
 * Parse the ini files. Create the .ooodl directory if necessary
 * Create the gtkrc file if necessary. Read the ooodl.rc file
 * (if it exists)
 */

static int do_init_files(void)
{
    char rcpath[PATH_MAX];
    char rc1path[PATH_MAX];
    int flg = 0;
    char *p;
    char *home;
    int have_hyph = 0;
    
    FILE *fp;
    
    if(NULL != (p = getenv("HOME")))
    {
        char *r;
        home = p;
        
        r = stpcpy(rc1path, p);
        p = stpcpy(rcpath, p);
        p = stpcpy(p, "/.ooodi");
        r = stpcpy(r, "/.ooodl");

            /* use new (0.3) names */
        if(0 != access(rcpath, W_OK))
        {
            if(0 == access(rc1path, W_OK))
            {
                rename (rc1path, rcpath);
            }
            else
            {
                    /* create if necessary */
                mkdir(rcpath, 0700);
            }
        }
        strcpy(rc1path, rcpath);
        
        *p++ = '/';
        strcpy(p, ".gtkrc2");
        
        if(0 != access(rcpath, R_OK))
        {
            fp = fopen(rcpath, "w");
            if(fp != NULL)
            {
                fputs(baserc, fp);
                fclose(fp);
            }
        }

        gtk_rc_add_default_file (rcpath);
        
        strcpy(p,"ooodi.rc");
        if(access(rcpath, W_OK) != 0)
        {
            strcpy(r, "/ooodl.rc");
            if(0 == access(rc1path, W_OK))
            {
                rename (rc1path, rcpath);
            }
        }

        if(NULL != (fp = fopen(rcpath, "r")))
        {
            char *q,buf[256];
            while(fgets(buf, sizeof(buf), fp))
            {
                flg++;
                if(1 == sscanf(buf, "proxy%*[ \t:=]%a[^\n]\n", &q))
                {
                    gblptr->proxy = q;
                }
                if(1 == sscanf(buf, "uri%*[ \t:=]%a[^\n]\n", &q))
                {
                    gblptr->uri = q;
                }
                if(1 == sscanf(buf, "user%*[ \t:=]%a[^\n]\n", &q))
                {
                    gblptr->pxuser = q;
                }
                if(1 == sscanf(buf, "browser%*[ \t:=]%a[^\n]\n", &q))
                {
                    gblptr->browser = q;
                }
                if(1 == sscanf(buf, "basedir%*[ \t:=]%a[^\n]\n", &q))
                {
                    gblptr->basedir = q;
                }
                if(1 == sscanf(buf, "keep_zips%*[ \t:=]%a[^\n]\n", &q))
                {
                    gblptr->keepzips = atoi(q);
                }
                if(1 == sscanf(buf, "use_local%*[ \t:=]%a[^\n]\n", &q))
                {
                    gblptr->uselocal = atoi(q);
                }
                if(1 == sscanf(buf, "use_hyphen%*[ \t:=]%a[^\n]\n", &q))
                {
                    have_hyph = 1;
                    gblptr->usehyph = atoi(q);
                }
                if(1 == sscanf(buf, "unlink_on_remove%*[ \t:=]%a[^\n]\n", &q))
                {
                    gblptr->unlink_on_remove = atoi(q);
                }
            }
            fclose(fp);
        }
        if(gblptr->uri == NULL)
            gblptr->uri = g_strdup(BASEURL);
	/* Make error checking easier later */
        if(gblptr->proxy == NULL)
            gblptr->proxy = g_strdup(""); 
        if(gblptr->pxuser == NULL)
            gblptr->pxuser = g_strdup(""); 
        if(gblptr->basedir == NULL)
            gblptr->basedir = g_strdup("."); 

        if(gblptr->browser == NULL)
        {
            char *names[] = {"firebird", "galeon", "mozilla", "netscape", NULL};
            char **pp, *path;
            char pname[PATH_MAX];
            if(NULL != (path = getenv("PATH")))
            {
                char *p1,*p2,*p3;
                int done = 0;

                for(pp = names; !done && *pp; pp++)
                {
                    char *ppath = g_strdup(path);
                    for(p1 = ppath; !done && (p2 = strtok(p1,":")) ; p1 = NULL)
                    {
                        p3 = stpcpy(pname,p2);
                        *p3++ = '/';
                        strcpy(p3, *pp);
                        if(0 == access(pname, X_OK))
                        {
                            gblptr->browser = g_strdup(*pp);
                            done = 1;
                        }
                    }
                    g_free(ppath);
                }
            }
        }
        if(gblptr->browser == NULL)
        {
            gblptr->browser = g_strdup("");                 
        }
        if(flg == 0)
        {
            *p = 0;
            gblptr->basedir = g_strdup(rcpath);
            if(have_hyph == 0)
            {
                gblptr->usehyph = 1;
            }
            
        }
    }
    return flg;
}

void setwinicon(GtkWidget *win)
{
    static GdkPixmap *icon;
    static GdkBitmap *mask;

    gtk_widget_realize (win);
    if(NULL == icon)
    {
        icon = gdk_pixmap_create_from_xpm_d (win->window,
                                             &mask,
                                             &win->style->bg[GTK_STATE_NORMAL],
                                             ooo_gulls);
    }
    gdk_window_set_icon(win->window, NULL, icon, mask);
    gdk_window_set_icon_name (win->window, "OOodi");
}

int main (int argc, char *argv[])
{
  OOODL_GBL_t gbl;
  int rc;
  poptContext ctx;
  char *oobase = NULL;
  unsigned char show_vers = 0;
  struct poptOption options[] = {
      POPT_AUTOHELP
      {"base", 'b', POPT_ARG_STRING, &oobase, 0,
       _("OOo SYSTEM installation base directory"), NULL},
      {"version", 'v', POPT_ARG_NONE, &show_vers, 0,
                 _("show version"), NULL},
      POPT_TABLEEND};


  ctx = poptGetContext(NULL, argc, (const char **)argv, options, 0);
  poptSetOtherOptionHelp(ctx, _("[OPTIONS...]"));
  while (poptGetNextOpt (ctx) > 0);
  
  poptGetArgs (ctx);
  poptFreeContext (ctx);

  if(show_vers)
  {
      puts(VERSION);
      exit(0);
  }
  
  gblptr = &gbl;
  
  memset(&gbl, 0, sizeof(gbl));
  
#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset(PACKAGE, "UTF-8");
  textdomain (PACKAGE);
#endif

  gtk_set_locale ();
  rc = do_init_files();
  gtk_init (&argc, &argv);

  xml = glade_xml_new(PACKAGE_DATA_DIR "/" "OOodi.glade", NULL, NULL);
  assert(xml != NULL);
  
  glade_xml_signal_autoconnect(xml);
  
  if(0 == geteuid() && oobase)
  {
      gbl.avail.path_shr = g_realloc(oobase,
                                     strlen(oobase)+sizeof("/share/dict/ooo"));
      strcat(gbl.avail.path_shr, "/share/dict/ooo");
  }
  
/*
 * We don't really need this with libglade, but it's easier than
 * fixing everything up
 */
  
  assert(NULL != (gbl.window = glade_xml_get_widget (xml, "ooo_window")));
  assert(NULL != (gbl.errwin = glade_xml_get_widget (xml, "errordialog")));
  assert(NULL != (gbl.about  = glade_xml_get_widget (xml, "about")));
  assert(NULL != (gbl.popmenu  = glade_xml_get_widget (xml, "popmenu")));
  assert(NULL != (gbl.props = glade_xml_get_widget (xml, "props")));
  assert(NULL != (gbl.warn = glade_xml_get_widget (xml, "warning")));

  if(strstr(gblptr->uri, XURL_0) ||
     strstr(gblptr->uri, XURL_1))
  {
      DisplayWarning(gbl.warn,
                     _("Dictionary Site"),
                     _("You appear to be using the old dictionary download site.\n\n"
                       "Please be advised that this may not have the latest dictionaries\n"
                       "and may not provide a dictionary service in the future.\n\n"
                       "Your preferences have been upgraded to the current download site:\n\n"
                       BASEURL "\n\n"
                       "ftp:// works as well\n")
                     );
      g_free(gblptr->uri);
      gblptr->uri = g_strdup(BASEURL);
      SaveConfigFile();
  }

  
  setup_windows(&gbl);

  parse_user_dicts(&gbl);
  {
      char *ap,availfn[PATH_MAX];
      ap = stpcpy(availfn, gbl.basedir);
      *ap++ = '/';
      strcpy(ap, AVAIL);
      gbl.flag = access(availfn, R_OK);
  }

  if(0 == rc)
  {
      DisplayWarning(gbl.warn,_("Configuration"),
                     _("Welcome to the OpenOffice.org Dictionary Installer.\n\n"
                       "You appear to be running this program for the first time. "
                       "Before you continue this program needs a list of avaliable dictionaries.\n\n"
                       "The next dialog will give you the option to download the list from the internet immediately, "
                       "configure a proxy before downloading or to use a local list of dictionaries."));
  }

  fix_buttons();
  gtk_widget_queue_resize (gbl.window);
  gtk_widget_show (gbl.window);
  seturlcolour();
  setbgcolour();
  
  gtk_init_add (loaddictlist, &gbl);

  gtk_main ();
  if(0 == rc)
  {
      SaveConfigFile();
  }
  return 0;
}

